@@ -1,7 +1,6 @@
 # -*- coding=utf-8 -*-
 from __future__ import absolute_import, print_function
 
-import copy
 import logging
 import operator
 import platform
@@ -11,24 +10,27 @@ from collections import defaultdict
 import attr
 import six
 from packaging.version import Version
-from vistir.compat import Path, lru_cache
 
-from .mixins import BaseFinder, BasePath
+from ..compat import Path, lru_cache
 from ..environment import ASDF_DATA_DIR, MYPY_RUNNING, PYENV_ROOT, SYSTEM_ARCH
 from ..exceptions import InvalidPythonVersion
 from ..utils import (
     RE_MATCHER,
     _filter_none,
     ensure_path,
+    expand_paths,
     get_python_version,
+    guess_company,
     is_in_path,
     looks_like_python,
     optional_instance_of,
     parse_asdf_version_order,
     parse_pyenv_version_order,
     parse_python_version,
+    path_is_pythoncore,
     unnest,
 )
+from .mixins import BaseFinder, BasePath
 
 if MYPY_RUNNING:
     from typing import (
@@ -44,9 +46,14 @@ if MYPY_RUNNING:
         Type,
         TypeVar,
         Iterator,
+        overload,
     )
     from .path import PathEntry
     from .._vendor.pep514tools.environment import Environment
+else:
+
+    def overload(f):
+        return f
 
 
 logger = logging.getLogger(__name__)
@@ -114,7 +121,8 @@ class PythonFinder(BaseFinder, BasePath):
                 versions[v] for v in parse_asdf_version_order() if v in versions
             ]
         for version in version_order:
-            version_paths.remove(version)
+            if version in version_paths:
+                version_paths.remove(version)
         if version_order:
             version_order += version_paths
         else:
@@ -228,11 +236,10 @@ class PythonFinder(BaseFinder, BasePath):
         # type: () -> DefaultDict[str, PathEntry]
         return self.pythons
 
+    @overload
     @classmethod
-    def create(
-        cls, root, sort_function, version_glob_path=None, ignore_unsupported=True
-    ):  # type: ignore
-        # type: (Type[PythonFinder], str, Callable, Optional[str], bool) -> PythonFinder
+    def create(cls, root, sort_function, version_glob_path=None, ignore_unsupported=True):
+        # type: (str, Callable, Optional[str], bool) -> PythonFinder
         root = ensure_path(root)
         if not version_glob_path:
             version_glob_path = "versions/*"
@@ -280,12 +287,10 @@ class PythonFinder(BaseFinder, BasePath):
             ]
         else:
             pythons = [sub_finder(path) for path in self.paths]
-        pythons = [p for p in pythons if p and p.is_python and p.as_python is not None]
+        pythons = expand_paths(pythons, True)
         version_sort = operator.attrgetter("as_python.version_sort")
         paths = [
-            p
-            for p in sorted(list(pythons), key=version_sort, reverse=True)
-            if p is not None
+            p for p in sorted(pythons, key=version_sort, reverse=True) if p is not None
         ]
         return paths
 
@@ -353,6 +358,7 @@ class PythonVersion(object):
     architecture = attr.ib(default=None)  # type: Optional[str]
     comes_from = attr.ib(default=None)  # type: Optional[PathEntry]
     executable = attr.ib(default=None)  # type: Optional[str]
+    company = attr.ib(default=None)  # type: Optional[str]
     name = attr.ib(default=None, type=str)
 
     def __getattribute__(self, key):
@@ -379,15 +385,18 @@ class PythonVersion(object):
 
     @property
     def version_sort(self):
-        # type: () -> Tuple[Optional[int], Optional[int], int, int]
+        # type: () -> Tuple[int, int, Optional[int], int, int]
         """
         A tuple for sorting against other instances of the same class.
 
-        Returns a tuple of the python version but includes a point for non-dev,
-        and a point for non-prerelease versions.  So released versions will have 2 points
-        for this value.  E.g. `(3, 6, 6, 2)` is a release, `(3, 6, 6, 1)` is a prerelease,
-        `(3, 6, 6, 0)` is a dev release, and `(3, 6, 6, 3)` is a postrelease.
+        Returns a tuple of the python version but includes points for core python,
+        non-dev,  and non-prerelease versions.  So released versions will have 2 points
+        for this value.  E.g. ``(1, 3, 6, 6, 2)`` is a release, ``(1, 3, 6, 6, 1)`` is a
+        prerelease, ``(1, 3, 6, 6, 0)`` is a dev release, and ``(1, 3, 6, 6, 3)`` is a
+        postrelease.  ``(0, 3, 7, 3, 2)`` represents a non-core python release, e.g. by
+        a repackager of python like Continuum.
         """
+        company_sort = 1 if (self.company and self.company == "PythonCore") else 0
         release_sort = 2
         if self.is_postrelease:
             release_sort = 3
@@ -397,7 +406,13 @@ class PythonVersion(object):
             release_sort = 0
         elif self.is_debug:
             release_sort = 1
-        return (self.major, self.minor, self.patch if self.patch else 0, release_sort)
+        return (
+            company_sort,
+            self.major,
+            self.minor,
+            self.patch if self.patch else 0,
+            release_sort,
+        )
 
     @property
     def version_tuple(self):
@@ -475,6 +490,7 @@ class PythonVersion(object):
             "is_devrelease": self.is_devrelease,
             "is_debug": self.is_debug,
             "version": self.version,
+            "company": self.company,
         }
 
     def update_metadata(self, metadata):
@@ -489,7 +505,7 @@ class PythonVersion(object):
 
         for key in metadata:
             try:
-                current_value = getattr(self, key)
+                _ = getattr(self, key)
             except AttributeError:
                 continue
             else:
@@ -534,8 +550,8 @@ class PythonVersion(object):
         return self.architecture
 
     @classmethod
-    def from_path(cls, path, name=None, ignore_unsupported=True):
-        # type: (Union[str, PathEntry], Optional[str], bool) -> PythonVersion
+    def from_path(cls, path, name=None, ignore_unsupported=True, company=None):
+        # type: (Union[str, PathEntry], Optional[str], bool, Optional[str]) -> PythonVersion
         """
         Parses a python version from a system path.
 
@@ -546,6 +562,7 @@ class PythonVersion(object):
         :type path: str or :class:`~pythonfinder.models.path.PathEntry` instance
         :param str name: Name of the python distribution in question
         :param bool ignore_unsupported: Whether to ignore or error on unsupported paths.
+        :param Optional[str] company: The company or vendor packaging the distribution.
         :return: An instance of a PythonVersion.
         :rtype: :class:`~pythonfinder.models.python.PythonVersion`
         """
@@ -578,6 +595,8 @@ class PythonVersion(object):
             instance_dict = cls.parse_executable(path.path.absolute().as_posix())
         if name is None:
             name = path_name
+        if company is None:
+            company = guess_company(path.path.as_posix())
         instance_dict.update(
             {"comes_from": path, "name": name, "executable": path.path.as_posix()}
         )
@@ -593,6 +612,8 @@ class PythonVersion(object):
             raise TypeError("Must pass a valid path to parse.")
         if not isinstance(path, six.string_types):
             path = path.as_posix()
+        # if not looks_like_python(path):
+        #     raise ValueError("Path %r does not look like a valid python path" % path)
         try:
             result_version = get_python_version(path)
         except Exception:
@@ -603,11 +624,13 @@ class PythonVersion(object):
         return result_dict
 
     @classmethod
-    def from_windows_launcher(cls, launcher_entry, name=None):
-        # type: (Environment, Optional[str]) -> PythonVersion
+    def from_windows_launcher(cls, launcher_entry, name=None, company=None):
+        # type: (Environment, Optional[str], Optional[str]) -> PythonVersion
         """Create a new PythonVersion instance from a Windows Launcher Entry
 
         :param launcher_entry: A python launcher environment object.
+        :param Optional[str] name: The name of the distribution.
+        :param Optional[str] company: The name of the distributing company.
         :return: An instance of a PythonVersion.
         :rtype: :class:`~pythonfinder.models.python.PythonVersion`
         """
@@ -622,6 +645,7 @@ class PythonVersion(object):
         exe_path = ensure_path(
             getattr(launcher_entry.info.install_path, "executable_path", default_path)
         )
+        company = getattr(launcher_entry, "company", guess_company(exe_path.as_posix()))
         creation_dict.update(
             {
                 "architecture": getattr(
@@ -629,6 +653,7 @@ class PythonVersion(object):
                 ),
                 "executable": exe_path,
                 "name": name,
+                "company": company,
             }
         )
         py_version = cls.create(**creation_dict)
@@ -656,7 +681,7 @@ class VersionMap(object):
         # type: (...) -> None
         version = entry.as_python  # type: PythonVersion
         if version:
-            entries = self.versions[version.version_tuple]
+            _ = self.versions[version.version_tuple]
             paths = {p.path for p in self.versions.get(version.version_tuple, [])}
             if entry.path not in paths:
                 self.versions[version.version_tuple].append(entry)
